package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TCoupon;
import com.haomostudio.so.service.TCouponService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TCoupon")
@Controller
public class TCouponController {

    @Autowired
    TCouponService tCouponService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TCoupon'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_coupons/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTCoupon(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "couponNo", required = false) String couponNo,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "discount", required = false) Integer discount,
            @RequestParam(value = "reduce", required = false) Integer reduce,
            @RequestParam(value = "full", required = false) Integer full,
            @RequestParam(value = "mark", required = false) String mark,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "total", required = false) Integer total,
            @RequestParam(value = "publishCount", required = false) Integer publishCount,
            @RequestParam(value = "publishStatus", required = false) Integer publishStatus,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "controlStatus", required = false) Integer controlStatus,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        TCoupon item = new TCoupon();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( couponNo != null ){
            item.setCouponNo(couponNo);
        }
        if( name != null ){
            item.setName(name);
        }
        if( type != null ){
            item.setType(type);
        }
        if( discount != null ){
            item.setDiscount(discount);
        }
        if( reduce != null ){
            item.setReduce(reduce);
        }
        if( full != null ){
            item.setFull(full);
        }
        if( mark != null ){
            item.setMark(mark);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( startDate != null ){
            item.setStartDate(Tools.convertStringToDate(startDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( endDate != null ){
            item.setEndDate(Tools.convertStringToDate(endDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( total != null ){
            item.setTotal(total);
        }
        if( publishCount != null ){
            item.setPublishCount(publishCount);
        }
        if( publishStatus != null ){
            item.setPublishStatus(publishStatus);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        if( controlStatus != null ){
            item.setControlStatus(controlStatus);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        tCouponService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tCouponService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TCoupon'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_coupons/{t_coupon_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTCoupon(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_coupon_id") Integer id
    ) {
        TCoupon item = tCouponService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tCouponService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TCoupon'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_coupons/{t_coupon_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTCoupon(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_coupon_id") Integer id,
            @RequestParam(value = "couponNo", required = false) String couponNo,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "discount", required = false) Integer discount,
            @RequestParam(value = "reduce", required = false) Integer reduce,
            @RequestParam(value = "full", required = false) Integer full,
            @RequestParam(value = "mark", required = false) String mark,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "total", required = false) Integer total,
            @RequestParam(value = "publishCount", required = false) Integer publishCount,
            @RequestParam(value = "publishStatus", required = false) Integer publishStatus,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "controlStatus", required = false) Integer controlStatus,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        TCoupon item = tCouponService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( couponNo != null ){
            item.setCouponNo(couponNo);
        }
        if( name != null ){
            item.setName(name);
        }
        if( type != null ){
            item.setType(type);
        }
        if( discount != null ){
            item.setDiscount(discount);
        }
        if( reduce != null ){
            item.setReduce(reduce);
        }
        if( full != null ){
            item.setFull(full);
        }
        if( mark != null ){
            item.setMark(mark);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( startDate != null ){
            item.setStartDate(Tools.convertStringToDate(startDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( endDate != null ){
            item.setEndDate(Tools.convertStringToDate(endDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( total != null ){
            item.setTotal(total);
        }
        if( publishCount != null ){
            item.setPublishCount(publishCount);
        }
        if( publishStatus != null ){
            item.setPublishStatus(publishStatus);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        if( controlStatus != null ){
            item.setControlStatus(controlStatus);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        tCouponService.update(item);

        return tCouponService.get(id);
    }

    @ApiOperation(value = "查询'TCoupon'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_coupons/{t_coupon_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCoupon(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_coupon_id") Integer id,
            @RequestParam(value = "couponNo", required = false) String couponNo,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "discount", required = false) Integer discount,
            @RequestParam(value = "reduce", required = false) Integer reduce,
            @RequestParam(value = "full", required = false) Integer full,
            @RequestParam(value = "mark", required = false) String mark,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "startDate", required = false) String startDate,
            @RequestParam(value = "endDate", required = false) String endDate,
            @RequestParam(value = "total", required = false) Integer total,
            @RequestParam(value = "publishCount", required = false) Integer publishCount,
            @RequestParam(value = "publishStatus", required = false) Integer publishStatus,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "controlStatus", required = false) Integer controlStatus,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        if(request.getMethod().equals("GET")){
            TCoupon item = tCouponService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TCoupon item = tCouponService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( couponNo != null ){
                item.setCouponNo(couponNo);
            }
            if( name != null ){
                item.setName(name);
            }
            if( type != null ){
                item.setType(type);
            }
            if( discount != null ){
                item.setDiscount(discount);
            }
            if( reduce != null ){
                item.setReduce(reduce);
            }
            if( full != null ){
                item.setFull(full);
            }
            if( mark != null ){
                item.setMark(mark);
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            if( startDate != null ){
                item.setStartDate(Tools.convertStringToDate(startDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( endDate != null ){
                item.setEndDate(Tools.convertStringToDate(endDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( total != null ){
                item.setTotal(total);
            }
            if( publishCount != null ){
                item.setPublishCount(publishCount);
            }
            if( publishStatus != null ){
                item.setPublishStatus(publishStatus);
            }
            if( adminUserId != null ){
                item.setAdminUserId(adminUserId);
            }
            if( controlStatus != null ){
                item.setControlStatus(controlStatus);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            tCouponService.update(item);

            return tCouponService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TCoupon item = tCouponService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tCouponService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TCoupon'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_coupons",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCoupons(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tCouponService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tCouponService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_coupons/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCouponsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TCoupon> list = tCouponService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TCoupon.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TCoupon'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_coupons/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTCoupons(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TCoupon> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TCoupon item = new TCoupon();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("couponNo")   ){
            item.setCouponNo(jsonObject.get("couponNo").toString());
        }
        if(jsonObject.containsKey("name")   ){
            item.setName(jsonObject.get("name").toString());
        }
        if(jsonObject.containsKey("type")   ){
            item.setType(Integer.valueOf(jsonObject.get("type").toString()));
        }
        if(jsonObject.containsKey("discount")   ){
            item.setDiscount(Integer.valueOf(jsonObject.get("discount").toString()));
        }
        if(jsonObject.containsKey("reduce")   ){
            item.setReduce(Integer.valueOf(jsonObject.get("reduce").toString()));
        }
        if(jsonObject.containsKey("full")   ){
            item.setFull(Integer.valueOf(jsonObject.get("full").toString()));
        }
        if(jsonObject.containsKey("mark")   ){
            item.setMark(jsonObject.get("mark").toString());
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("startDate")   ){
            item.setStartDate(Tools.convertStringToDate(jsonObject.get("startDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("endDate")   ){
            item.setEndDate(Tools.convertStringToDate(jsonObject.get("endDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("total")   ){
            item.setTotal(Integer.valueOf(jsonObject.get("total").toString()));
        }
        if(jsonObject.containsKey("publishCount")   ){
            item.setPublishCount(Integer.valueOf(jsonObject.get("publishCount").toString()));
        }
        if(jsonObject.containsKey("publishStatus")   ){
            item.setPublishStatus(Integer.valueOf(jsonObject.get("publishStatus").toString()));
        }
        if(jsonObject.containsKey("adminUserId")   ){
            item.setAdminUserId(Integer.valueOf(jsonObject.get("adminUserId").toString()));
        }
        if(jsonObject.containsKey("controlStatus")   ){
            item.setControlStatus(Integer.valueOf(jsonObject.get("controlStatus").toString()));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
             tCouponService.create(item);
             lists.add(tCouponService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TCoupon'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_coupons/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTCoupon(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TCoupon item = tCouponService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tCouponService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
