package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TCreditRegisterLog;
import com.haomostudio.so.service.TCreditRegisterLogService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TCreditRegisterLog")
@Controller
public class TCreditRegisterLogController {

    @Autowired
    TCreditRegisterLogService tCreditRegisterLogService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TCreditRegisterLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_credit_register_logs/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTCreditRegisterLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "creditRuleDetailId", required = false) Integer creditRuleDetailId,
            @RequestParam(value = "creditRuleDetailItem", required = false) String creditRuleDetailItem,
            @RequestParam(value = "operate", required = false) Integer operate,
            @RequestParam(value = "creditPoint", required = false) Integer creditPoint,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "remark", required = false) String remark
    ) {
        TCreditRegisterLog item = new TCreditRegisterLog();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( userId != null ){
            item.setUserId(userId);
        }
        if( creditRuleDetailId != null ){
            item.setCreditRuleDetailId(creditRuleDetailId);
        }
        if( creditRuleDetailItem != null ){
            item.setCreditRuleDetailItem(creditRuleDetailItem);
        }
        if( operate != null ){
            item.setOperate(operate);
        }
        if( creditPoint != null ){
            item.setCreditPoint(creditPoint);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        if( remark != null ){
            item.setRemark(remark);
        }
        tCreditRegisterLogService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tCreditRegisterLogService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TCreditRegisterLog'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_credit_register_logs/{t_credit_register_log_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTCreditRegisterLog(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_credit_register_log_id") Integer id
    ) {
        TCreditRegisterLog item = tCreditRegisterLogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tCreditRegisterLogService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TCreditRegisterLog'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_credit_register_logs/{t_credit_register_log_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTCreditRegisterLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_credit_register_log_id") Integer id,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "creditRuleDetailId", required = false) Integer creditRuleDetailId,
            @RequestParam(value = "creditRuleDetailItem", required = false) String creditRuleDetailItem,
            @RequestParam(value = "operate", required = false) Integer operate,
            @RequestParam(value = "creditPoint", required = false) Integer creditPoint,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "remark", required = false) String remark
    ) {
        TCreditRegisterLog item = tCreditRegisterLogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( userId != null ){
            item.setUserId(userId);
        }
        if( creditRuleDetailId != null ){
            item.setCreditRuleDetailId(creditRuleDetailId);
        }
        if( creditRuleDetailItem != null ){
            item.setCreditRuleDetailItem(creditRuleDetailItem);
        }
        if( operate != null ){
            item.setOperate(operate);
        }
        if( creditPoint != null ){
            item.setCreditPoint(creditPoint);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        if( remark != null ){
            item.setRemark(remark);
        }
        tCreditRegisterLogService.update(item);

        return tCreditRegisterLogService.get(id);
    }

    @ApiOperation(value = "查询'TCreditRegisterLog'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_credit_register_logs/{t_credit_register_log_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCreditRegisterLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_credit_register_log_id") Integer id,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "creditRuleDetailId", required = false) Integer creditRuleDetailId,
            @RequestParam(value = "creditRuleDetailItem", required = false) String creditRuleDetailItem,
            @RequestParam(value = "operate", required = false) Integer operate,
            @RequestParam(value = "creditPoint", required = false) Integer creditPoint,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "remark", required = false) String remark
    ) {
        if(request.getMethod().equals("GET")){
            TCreditRegisterLog item = tCreditRegisterLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TCreditRegisterLog item = tCreditRegisterLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( userId != null ){
                item.setUserId(userId);
            }
            if( creditRuleDetailId != null ){
                item.setCreditRuleDetailId(creditRuleDetailId);
            }
            if( creditRuleDetailItem != null ){
                item.setCreditRuleDetailItem(creditRuleDetailItem);
            }
            if( operate != null ){
                item.setOperate(operate);
            }
            if( creditPoint != null ){
                item.setCreditPoint(creditPoint);
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            if( adminUserId != null ){
                item.setAdminUserId(adminUserId);
            }
            if( remark != null ){
                item.setRemark(remark);
            }
            tCreditRegisterLogService.update(item);

            return tCreditRegisterLogService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TCreditRegisterLog item = tCreditRegisterLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tCreditRegisterLogService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TCreditRegisterLog'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_credit_register_logs",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCreditRegisterLogs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tCreditRegisterLogService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tCreditRegisterLogService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_credit_register_logs/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCreditRegisterLogsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TCreditRegisterLog> list = tCreditRegisterLogService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TCreditRegisterLog.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TCreditRegisterLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_credit_register_logs/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTCreditRegisterLogs(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TCreditRegisterLog> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TCreditRegisterLog item = new TCreditRegisterLog();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("userId")   ){
            item.setUserId(Integer.valueOf(jsonObject.get("userId").toString()));
        }
        if(jsonObject.containsKey("creditRuleDetailId")   ){
            item.setCreditRuleDetailId(Integer.valueOf(jsonObject.get("creditRuleDetailId").toString()));
        }
        if(jsonObject.containsKey("creditRuleDetailItem")   ){
            item.setCreditRuleDetailItem(jsonObject.get("creditRuleDetailItem").toString());
        }
        if(jsonObject.containsKey("operate")   ){
            item.setOperate(Integer.valueOf(jsonObject.get("operate").toString()));
        }
        if(jsonObject.containsKey("creditPoint")   ){
            item.setCreditPoint(Integer.valueOf(jsonObject.get("creditPoint").toString()));
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("adminUserId")   ){
            item.setAdminUserId(Integer.valueOf(jsonObject.get("adminUserId").toString()));
        }
        if(jsonObject.containsKey("remark")   ){
            item.setRemark(jsonObject.get("remark").toString());
        }
             tCreditRegisterLogService.create(item);
             lists.add(tCreditRegisterLogService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TCreditRegisterLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_credit_register_logs/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTCreditRegisterLog(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TCreditRegisterLog item = tCreditRegisterLogService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tCreditRegisterLogService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
