package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TMultimedia;
import com.haomostudio.so.service.TMultimediaService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TMultimedia")
@Controller
public class TMultimediaController {

    @Autowired
    TMultimediaService tMultimediaService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TMultimedia'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_multimedias/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTMultimedia(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "fileName", required = false) String fileName,
            @RequestParam(value = "fileUrl", required = false) String fileUrl,
            @RequestParam(value = "fileType", required = false) Integer fileType,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "stayTime", required = false) Integer stayTime,
            @RequestParam(value = "openState", required = false) Integer openState,
            @RequestParam(value = "playOrder", required = false) Integer playOrder,
            @RequestParam(value = "createTime", required = false) String createTime,
            @RequestParam(value = "delState", required = false) Integer delState,
            @RequestParam(value = "deleteTime", required = false) String deleteTime,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        TMultimedia item = new TMultimedia();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( fileName != null ){
            item.setFileName(fileName);
        }
        if( fileUrl != null ){
            item.setFileUrl(fileUrl);
        }
        if( fileType != null ){
            item.setFileType(fileType);
        }
        if( startTime != null ){
            item.setStartTime(Tools.convertStringToDate(startTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( endTime != null ){
            item.setEndTime(Tools.convertStringToDate(endTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( stayTime != null ){
            item.setStayTime(stayTime);
        }
        if( openState != null ){
            item.setOpenState(openState);
        }
        if( playOrder != null ){
            item.setPlayOrder(playOrder);
        }
            item.setCreateTime(new Date());
        if( delState != null ){
            item.setDelState(delState);
        }
        if( deleteTime != null ){
            item.setDeleteTime(Tools.convertStringToDate(deleteTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        tMultimediaService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tMultimediaService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TMultimedia'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_multimedias/{t_multimedia_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTMultimedia(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_multimedia_id") Integer id
    ) {
        TMultimedia item = tMultimediaService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tMultimediaService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TMultimedia'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_multimedias/{t_multimedia_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTMultimedia(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_multimedia_id") Integer id,
            @RequestParam(value = "fileName", required = false) String fileName,
            @RequestParam(value = "fileUrl", required = false) String fileUrl,
            @RequestParam(value = "fileType", required = false) Integer fileType,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "stayTime", required = false) Integer stayTime,
            @RequestParam(value = "openState", required = false) Integer openState,
            @RequestParam(value = "playOrder", required = false) Integer playOrder,
            @RequestParam(value = "delState", required = false) Integer delState,
            @RequestParam(value = "deleteTime", required = false) String deleteTime,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        TMultimedia item = tMultimediaService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( fileName != null ){
            item.setFileName(fileName);
        }
        if( fileUrl != null ){
            item.setFileUrl(fileUrl);
        }
        if( fileType != null ){
            item.setFileType(fileType);
        }
        if( startTime != null ){
            item.setStartTime(Tools.convertStringToDate(startTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( endTime != null ){
            item.setEndTime(Tools.convertStringToDate(endTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( stayTime != null ){
            item.setStayTime(stayTime);
        }
        if( openState != null ){
            item.setOpenState(openState);
        }
        if( playOrder != null ){
            item.setPlayOrder(playOrder);
        }
        if( delState != null ){
            item.setDelState(delState);
        }
        if( deleteTime != null ){
            item.setDeleteTime(Tools.convertStringToDate(deleteTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        tMultimediaService.update(item);

        return tMultimediaService.get(id);
    }

    @ApiOperation(value = "查询'TMultimedia'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_multimedias/{t_multimedia_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTMultimedia(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_multimedia_id") Integer id,
            @RequestParam(value = "fileName", required = false) String fileName,
            @RequestParam(value = "fileUrl", required = false) String fileUrl,
            @RequestParam(value = "fileType", required = false) Integer fileType,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "stayTime", required = false) Integer stayTime,
            @RequestParam(value = "openState", required = false) Integer openState,
            @RequestParam(value = "playOrder", required = false) Integer playOrder,
            @RequestParam(value = "delState", required = false) Integer delState,
            @RequestParam(value = "deleteTime", required = false) String deleteTime,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        if(request.getMethod().equals("GET")){
            TMultimedia item = tMultimediaService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TMultimedia item = tMultimediaService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( fileName != null ){
                item.setFileName(fileName);
            }
            if( fileUrl != null ){
                item.setFileUrl(fileUrl);
            }
            if( fileType != null ){
                item.setFileType(fileType);
            }
            if( startTime != null ){
                item.setStartTime(Tools.convertStringToDate(startTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( endTime != null ){
                item.setEndTime(Tools.convertStringToDate(endTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( stayTime != null ){
                item.setStayTime(stayTime);
            }
            if( openState != null ){
                item.setOpenState(openState);
            }
            if( playOrder != null ){
                item.setPlayOrder(playOrder);
            }
            if( delState != null ){
                item.setDelState(delState);
            }
            if( deleteTime != null ){
                item.setDeleteTime(Tools.convertStringToDate(deleteTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( adminUserId != null ){
                item.setAdminUserId(adminUserId);
            }
            tMultimediaService.update(item);

            return tMultimediaService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TMultimedia item = tMultimediaService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tMultimediaService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TMultimedia'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_multimedias",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTMultimedias(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tMultimediaService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tMultimediaService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_multimedias/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTMultimediasId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TMultimedia> list = tMultimediaService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TMultimedia.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TMultimedia'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_multimedias/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTMultimedias(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TMultimedia> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TMultimedia item = new TMultimedia();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("fileName")   ){
            item.setFileName(jsonObject.get("fileName").toString());
        }
        if(jsonObject.containsKey("fileUrl")   ){
            item.setFileUrl(jsonObject.get("fileUrl").toString());
        }
        if(jsonObject.containsKey("fileType")   ){
            item.setFileType(Integer.valueOf(jsonObject.get("fileType").toString()));
        }
        if(jsonObject.containsKey("startTime")   ){
            item.setStartTime(Tools.convertStringToDate(jsonObject.get("startTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("endTime")   ){
            item.setEndTime(Tools.convertStringToDate(jsonObject.get("endTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("stayTime")   ){
            item.setStayTime(Integer.valueOf(jsonObject.get("stayTime").toString()));
        }
        if(jsonObject.containsKey("openState")   ){
            item.setOpenState(Integer.valueOf(jsonObject.get("openState").toString()));
        }
        if(jsonObject.containsKey("playOrder")   ){
            item.setPlayOrder(Integer.valueOf(jsonObject.get("playOrder").toString()));
        }
            item.setCreateTime(new Date());
        if(jsonObject.containsKey("delState")   ){
            item.setDelState(Integer.valueOf(jsonObject.get("delState").toString()));
        }
        if(jsonObject.containsKey("deleteTime")   ){
            item.setDeleteTime(Tools.convertStringToDate(jsonObject.get("deleteTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("adminUserId")   ){
            item.setAdminUserId(Integer.valueOf(jsonObject.get("adminUserId").toString()));
        }
             tMultimediaService.create(item);
             lists.add(tMultimediaService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TMultimedia'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_multimedias/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTMultimedia(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TMultimedia item = tMultimediaService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tMultimediaService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
