package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TOuth2User;
import com.haomostudio.so.service.TOuth2UserService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TOuth2User")
@Controller
public class TOuth2UserController {

    @Autowired
    TOuth2UserService tOuth2UserService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TOuth2User'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_outh2_users/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTOuth2User(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "nickname", required = false) String nickname,
            @RequestParam(value = "headImgUrl", required = false) String headImgUrl,
            @RequestParam(value = "unionId", required = false) String unionId,
            @RequestParam(value = "openId", required = false) String openId,
            @RequestParam(value = "data", required = false) String data,
            @RequestParam(value = "channel", required = false) String channel,
            @RequestParam(value = "activityCode", required = false) String activityCode,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "sex", required = false) Integer sex,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        TOuth2User item = new TOuth2User();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( nickname != null ){
            item.setNickname(nickname);
        }
        if( headImgUrl != null ){
            item.setHeadImgUrl(headImgUrl);
        }
        if( unionId != null ){
            item.setUnionId(unionId);
        }
        if( openId != null ){
            item.setOpenId(openId);
        }
        if( data != null ){
            item.setData(data);
        }
        if( channel != null ){
            item.setChannel(channel);
        }
        if( activityCode != null ){
            item.setActivityCode(activityCode);
        }
        if( userId != null ){
            item.setUserId(userId);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( sex != null ){
            item.setSex(sex);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        tOuth2UserService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tOuth2UserService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TOuth2User'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_outh2_users/{t_outh2_user_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTOuth2User(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_outh2_user_id") Integer id
    ) {
        TOuth2User item = tOuth2UserService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tOuth2UserService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TOuth2User'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_outh2_users/{t_outh2_user_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTOuth2User(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_outh2_user_id") Integer id,
            @RequestParam(value = "nickname", required = false) String nickname,
            @RequestParam(value = "headImgUrl", required = false) String headImgUrl,
            @RequestParam(value = "unionId", required = false) String unionId,
            @RequestParam(value = "openId", required = false) String openId,
            @RequestParam(value = "data", required = false) String data,
            @RequestParam(value = "channel", required = false) String channel,
            @RequestParam(value = "activityCode", required = false) String activityCode,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "sex", required = false) Integer sex,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        TOuth2User item = tOuth2UserService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( nickname != null ){
            item.setNickname(nickname);
        }
        if( headImgUrl != null ){
            item.setHeadImgUrl(headImgUrl);
        }
        if( unionId != null ){
            item.setUnionId(unionId);
        }
        if( openId != null ){
            item.setOpenId(openId);
        }
        if( data != null ){
            item.setData(data);
        }
        if( channel != null ){
            item.setChannel(channel);
        }
        if( activityCode != null ){
            item.setActivityCode(activityCode);
        }
        if( userId != null ){
            item.setUserId(userId);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( sex != null ){
            item.setSex(sex);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        tOuth2UserService.update(item);

        return tOuth2UserService.get(id);
    }

    @ApiOperation(value = "查询'TOuth2User'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_outh2_users/{t_outh2_user_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTOuth2User(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_outh2_user_id") Integer id,
            @RequestParam(value = "nickname", required = false) String nickname,
            @RequestParam(value = "headImgUrl", required = false) String headImgUrl,
            @RequestParam(value = "unionId", required = false) String unionId,
            @RequestParam(value = "openId", required = false) String openId,
            @RequestParam(value = "data", required = false) String data,
            @RequestParam(value = "channel", required = false) String channel,
            @RequestParam(value = "activityCode", required = false) String activityCode,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "sex", required = false) Integer sex,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        if(request.getMethod().equals("GET")){
            TOuth2User item = tOuth2UserService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TOuth2User item = tOuth2UserService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( nickname != null ){
                item.setNickname(nickname);
            }
            if( headImgUrl != null ){
                item.setHeadImgUrl(headImgUrl);
            }
            if( unionId != null ){
                item.setUnionId(unionId);
            }
            if( openId != null ){
                item.setOpenId(openId);
            }
            if( data != null ){
                item.setData(data);
            }
            if( channel != null ){
                item.setChannel(channel);
            }
            if( activityCode != null ){
                item.setActivityCode(activityCode);
            }
            if( userId != null ){
                item.setUserId(userId);
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            if( updated != null ){
                item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
            }
            if( sex != null ){
                item.setSex(sex);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            tOuth2UserService.update(item);

            return tOuth2UserService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TOuth2User item = tOuth2UserService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tOuth2UserService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TOuth2User'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_outh2_users",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTOuth2Users(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tOuth2UserService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tOuth2UserService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_outh2_users/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTOuth2UsersId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TOuth2User> list = tOuth2UserService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TOuth2User.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TOuth2User'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_outh2_users/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTOuth2Users(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TOuth2User> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TOuth2User item = new TOuth2User();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("nickname")   ){
            item.setNickname(jsonObject.get("nickname").toString());
        }
        if(jsonObject.containsKey("headImgUrl")   ){
            item.setHeadImgUrl(jsonObject.get("headImgUrl").toString());
        }
        if(jsonObject.containsKey("unionId")   ){
            item.setUnionId(jsonObject.get("unionId").toString());
        }
        if(jsonObject.containsKey("openId")   ){
            item.setOpenId(jsonObject.get("openId").toString());
        }
        if(jsonObject.containsKey("data")   ){
            item.setData(jsonObject.get("data").toString());
        }
        if(jsonObject.containsKey("channel")   ){
            item.setChannel(jsonObject.get("channel").toString());
        }
        if(jsonObject.containsKey("activityCode")   ){
            item.setActivityCode(jsonObject.get("activityCode").toString());
        }
        if(jsonObject.containsKey("userId")   ){
            item.setUserId(Integer.valueOf(jsonObject.get("userId").toString()));
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("updated")   ){
            item.setUpdated(Tools.convertStringToDate(jsonObject.get("updated").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("sex")   ){
            item.setSex(Integer.valueOf(jsonObject.get("sex").toString()));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
             tOuth2UserService.create(item);
             lists.add(tOuth2UserService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TOuth2User'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_outh2_users/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTOuth2User(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TOuth2User item = tOuth2UserService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tOuth2UserService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
