package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformActivityShowConfig;
import com.haomostudio.so.service.TPlatformActivityShowConfigService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformActivityShowConfig")
@Controller
public class TPlatformActivityShowConfigController {

    @Autowired
    TPlatformActivityShowConfigService tPlatformActivityShowConfigService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformActivityShowConfig'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_show_configs/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformActivityShowConfig(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "showDate", required = false) String showDate,
            @RequestParam(value = "hiddenDate", required = false) String hiddenDate,
            @RequestParam(value = "notBeginPrompt", required = false) String notBeginPrompt,
            @RequestParam(value = "endPrompt", required = false) String endPrompt,
            @RequestParam(value = "platformActivityId", required = false) Integer platformActivityId
    ) {
        TPlatformActivityShowConfig item = new TPlatformActivityShowConfig();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( type != null ){
            item.setType(type);
        }
        if( showDate != null ){
            item.setShowDate(Tools.convertStringToDate(showDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( hiddenDate != null ){
            item.setHiddenDate(Tools.convertStringToDate(hiddenDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( notBeginPrompt != null ){
            item.setNotBeginPrompt(notBeginPrompt);
        }
        if( endPrompt != null ){
            item.setEndPrompt(endPrompt);
        }
        if( platformActivityId != null ){
            item.setPlatformActivityId(platformActivityId);
        }
        tPlatformActivityShowConfigService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformActivityShowConfigService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformActivityShowConfig'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_activity_show_configs/{t_platform_activity_show_config_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformActivityShowConfig(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_show_config_id") Integer id
    ) {
        TPlatformActivityShowConfig item = tPlatformActivityShowConfigService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformActivityShowConfigService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformActivityShowConfig'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_show_configs/{t_platform_activity_show_config_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformActivityShowConfig(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_show_config_id") Integer id,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "showDate", required = false) String showDate,
            @RequestParam(value = "hiddenDate", required = false) String hiddenDate,
            @RequestParam(value = "notBeginPrompt", required = false) String notBeginPrompt,
            @RequestParam(value = "endPrompt", required = false) String endPrompt,
            @RequestParam(value = "platformActivityId", required = false) Integer platformActivityId
    ) {
        TPlatformActivityShowConfig item = tPlatformActivityShowConfigService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( type != null ){
            item.setType(type);
        }
        if( showDate != null ){
            item.setShowDate(Tools.convertStringToDate(showDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( hiddenDate != null ){
            item.setHiddenDate(Tools.convertStringToDate(hiddenDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( notBeginPrompt != null ){
            item.setNotBeginPrompt(notBeginPrompt);
        }
        if( endPrompt != null ){
            item.setEndPrompt(endPrompt);
        }
        if( platformActivityId != null ){
            item.setPlatformActivityId(platformActivityId);
        }
        tPlatformActivityShowConfigService.update(item);

        return tPlatformActivityShowConfigService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformActivityShowConfig'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_activity_show_configs/{t_platform_activity_show_config_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityShowConfig(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_show_config_id") Integer id,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "showDate", required = false) String showDate,
            @RequestParam(value = "hiddenDate", required = false) String hiddenDate,
            @RequestParam(value = "notBeginPrompt", required = false) String notBeginPrompt,
            @RequestParam(value = "endPrompt", required = false) String endPrompt,
            @RequestParam(value = "platformActivityId", required = false) Integer platformActivityId
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformActivityShowConfig item = tPlatformActivityShowConfigService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformActivityShowConfig item = tPlatformActivityShowConfigService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( type != null ){
                item.setType(type);
            }
            if( showDate != null ){
                item.setShowDate(Tools.convertStringToDate(showDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( hiddenDate != null ){
                item.setHiddenDate(Tools.convertStringToDate(hiddenDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( notBeginPrompt != null ){
                item.setNotBeginPrompt(notBeginPrompt);
            }
            if( endPrompt != null ){
                item.setEndPrompt(endPrompt);
            }
            if( platformActivityId != null ){
                item.setPlatformActivityId(platformActivityId);
            }
            tPlatformActivityShowConfigService.update(item);

            return tPlatformActivityShowConfigService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformActivityShowConfig item = tPlatformActivityShowConfigService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformActivityShowConfigService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformActivityShowConfig'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_activity_show_configs",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityShowConfigs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformActivityShowConfigService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformActivityShowConfigService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_activity_show_configs/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityShowConfigsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformActivityShowConfig> list = tPlatformActivityShowConfigService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformActivityShowConfig.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformActivityShowConfig'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_show_configs/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformActivityShowConfigs(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformActivityShowConfig> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformActivityShowConfig item = new TPlatformActivityShowConfig();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("type")   ){
            item.setType(Integer.valueOf(jsonObject.get("type").toString()));
        }
        if(jsonObject.containsKey("showDate")   ){
            item.setShowDate(Tools.convertStringToDate(jsonObject.get("showDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("hiddenDate")   ){
            item.setHiddenDate(Tools.convertStringToDate(jsonObject.get("hiddenDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("notBeginPrompt")   ){
            item.setNotBeginPrompt(jsonObject.get("notBeginPrompt").toString());
        }
        if(jsonObject.containsKey("endPrompt")   ){
            item.setEndPrompt(jsonObject.get("endPrompt").toString());
        }
        if(jsonObject.containsKey("platformActivityId")   ){
            item.setPlatformActivityId(Integer.valueOf(jsonObject.get("platformActivityId").toString()));
        }
             tPlatformActivityShowConfigService.create(item);
             lists.add(tPlatformActivityShowConfigService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformActivityShowConfig'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_activity_show_configs/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformActivityShowConfig(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformActivityShowConfig item = tPlatformActivityShowConfigService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformActivityShowConfigService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
