package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformActivityTime;
import com.haomostudio.so.service.TPlatformActivityTimeService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformActivityTime")
@Controller
public class TPlatformActivityTimeController {

    @Autowired
    TPlatformActivityTimeService tPlatformActivityTimeService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformActivityTime'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_times/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformActivityTime(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "activityName", required = false) String activityName,
            @RequestParam(value = "infoTitle", required = false) String infoTitle,
            @RequestParam(value = "couponTitle", required = false) String couponTitle,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "created", required = false) String created
    ) {
        TPlatformActivityTime item = new TPlatformActivityTime();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( activityName != null ){
            item.setActivityName(activityName);
        }
        if( infoTitle != null ){
            item.setInfoTitle(infoTitle);
        }
        if( couponTitle != null ){
            item.setCouponTitle(couponTitle);
        }
        if( startTime != null ){
            item.setStartTime(Tools.convertStringToDate(startTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( endTime != null ){
            item.setEndTime(Tools.convertStringToDate(endTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        tPlatformActivityTimeService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformActivityTimeService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformActivityTime'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_activity_times/{t_platform_activity_time_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformActivityTime(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_time_id") Integer id
    ) {
        TPlatformActivityTime item = tPlatformActivityTimeService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformActivityTimeService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformActivityTime'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_times/{t_platform_activity_time_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformActivityTime(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_time_id") Integer id,
            @RequestParam(value = "activityName", required = false) String activityName,
            @RequestParam(value = "infoTitle", required = false) String infoTitle,
            @RequestParam(value = "couponTitle", required = false) String couponTitle,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "created", required = false) String created
    ) {
        TPlatformActivityTime item = tPlatformActivityTimeService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( activityName != null ){
            item.setActivityName(activityName);
        }
        if( infoTitle != null ){
            item.setInfoTitle(infoTitle);
        }
        if( couponTitle != null ){
            item.setCouponTitle(couponTitle);
        }
        if( startTime != null ){
            item.setStartTime(Tools.convertStringToDate(startTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( endTime != null ){
            item.setEndTime(Tools.convertStringToDate(endTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        tPlatformActivityTimeService.update(item);

        return tPlatformActivityTimeService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformActivityTime'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_activity_times/{t_platform_activity_time_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityTime(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_time_id") Integer id,
            @RequestParam(value = "activityName", required = false) String activityName,
            @RequestParam(value = "infoTitle", required = false) String infoTitle,
            @RequestParam(value = "couponTitle", required = false) String couponTitle,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "created", required = false) String created
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformActivityTime item = tPlatformActivityTimeService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformActivityTime item = tPlatformActivityTimeService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( activityName != null ){
                item.setActivityName(activityName);
            }
            if( infoTitle != null ){
                item.setInfoTitle(infoTitle);
            }
            if( couponTitle != null ){
                item.setCouponTitle(couponTitle);
            }
            if( startTime != null ){
                item.setStartTime(Tools.convertStringToDate(startTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( endTime != null ){
                item.setEndTime(Tools.convertStringToDate(endTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            tPlatformActivityTimeService.update(item);

            return tPlatformActivityTimeService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformActivityTime item = tPlatformActivityTimeService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformActivityTimeService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformActivityTime'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_activity_times",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityTimes(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformActivityTimeService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformActivityTimeService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_activity_times/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityTimesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformActivityTime> list = tPlatformActivityTimeService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformActivityTime.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformActivityTime'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_times/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformActivityTimes(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformActivityTime> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformActivityTime item = new TPlatformActivityTime();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("activityName")   ){
            item.setActivityName(jsonObject.get("activityName").toString());
        }
        if(jsonObject.containsKey("infoTitle")   ){
            item.setInfoTitle(jsonObject.get("infoTitle").toString());
        }
        if(jsonObject.containsKey("couponTitle")   ){
            item.setCouponTitle(jsonObject.get("couponTitle").toString());
        }
        if(jsonObject.containsKey("startTime")   ){
            item.setStartTime(Tools.convertStringToDate(jsonObject.get("startTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("endTime")   ){
            item.setEndTime(Tools.convertStringToDate(jsonObject.get("endTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
             tPlatformActivityTimeService.create(item);
             lists.add(tPlatformActivityTimeService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformActivityTime'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_activity_times/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformActivityTime(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformActivityTime item = tPlatformActivityTimeService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformActivityTimeService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
