package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformFeedbackImgs;
import com.haomostudio.so.service.TPlatformFeedbackImgsService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformFeedbackImgs")
@Controller
public class TPlatformFeedbackImgsController {

    @Autowired
    TPlatformFeedbackImgsService tPlatformFeedbackImgsService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformFeedbackImgs'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_feedback_imgses/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformFeedbackImgs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "imgPath", required = false) String imgPath,
            @RequestParam(value = "platformFeedbackId", required = false) Integer platformFeedbackId
    ) {
        TPlatformFeedbackImgs item = new TPlatformFeedbackImgs();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( imgPath != null ){
            item.setImgPath(imgPath);
        }
        if( platformFeedbackId != null ){
            item.setPlatformFeedbackId(platformFeedbackId);
        }
        tPlatformFeedbackImgsService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformFeedbackImgsService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformFeedbackImgs'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_feedback_imgses/{t_platform_feedback_imgs_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformFeedbackImgs(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_feedback_imgs_id") Integer id
    ) {
        TPlatformFeedbackImgs item = tPlatformFeedbackImgsService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformFeedbackImgsService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformFeedbackImgs'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_feedback_imgses/{t_platform_feedback_imgs_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformFeedbackImgs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_feedback_imgs_id") Integer id,
            @RequestParam(value = "imgPath", required = false) String imgPath,
            @RequestParam(value = "platformFeedbackId", required = false) Integer platformFeedbackId
    ) {
        TPlatformFeedbackImgs item = tPlatformFeedbackImgsService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( imgPath != null ){
            item.setImgPath(imgPath);
        }
        if( platformFeedbackId != null ){
            item.setPlatformFeedbackId(platformFeedbackId);
        }
        tPlatformFeedbackImgsService.update(item);

        return tPlatformFeedbackImgsService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformFeedbackImgs'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_feedback_imgses/{t_platform_feedback_imgs_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformFeedbackImgs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_feedback_imgs_id") Integer id,
            @RequestParam(value = "imgPath", required = false) String imgPath,
            @RequestParam(value = "platformFeedbackId", required = false) Integer platformFeedbackId
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformFeedbackImgs item = tPlatformFeedbackImgsService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformFeedbackImgs item = tPlatformFeedbackImgsService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( imgPath != null ){
                item.setImgPath(imgPath);
            }
            if( platformFeedbackId != null ){
                item.setPlatformFeedbackId(platformFeedbackId);
            }
            tPlatformFeedbackImgsService.update(item);

            return tPlatformFeedbackImgsService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformFeedbackImgs item = tPlatformFeedbackImgsService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformFeedbackImgsService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformFeedbackImgs'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_feedback_imgses",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformFeedbackImgses(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformFeedbackImgsService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformFeedbackImgsService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_feedback_imgses/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformFeedbackImgsesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformFeedbackImgs> list = tPlatformFeedbackImgsService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformFeedbackImgs.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformFeedbackImgs'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_feedback_imgses/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformFeedbackImgses(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformFeedbackImgs> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformFeedbackImgs item = new TPlatformFeedbackImgs();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("imgPath")   ){
            item.setImgPath(jsonObject.get("imgPath").toString());
        }
        if(jsonObject.containsKey("platformFeedbackId")   ){
            item.setPlatformFeedbackId(Integer.valueOf(jsonObject.get("platformFeedbackId").toString()));
        }
             tPlatformFeedbackImgsService.create(item);
             lists.add(tPlatformFeedbackImgsService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformFeedbackImgs'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_feedback_imgses/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformFeedbackImgs(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformFeedbackImgs item = tPlatformFeedbackImgsService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformFeedbackImgsService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
