package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TSupervisor;
import com.haomostudio.so.service.TSupervisorService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TSupervisor")
@Controller
public class TSupervisorController {

    @Autowired
    TSupervisorService tSupervisorService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TSupervisor'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_supervisors/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTSupervisor(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "salt", required = false) String salt,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "modifiedDate", required = false) String modifiedDate,
            @RequestParam(value = "role", required = false) Integer role,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "lastLoginDate", required = false) String lastLoginDate
    ) {
        TSupervisor item = new TSupervisor();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( name != null ){
            item.setName(name);
        }
        if( username != null ){
            item.setUsername(username);
        }
        if( password != null ){
            item.setPassword(password);
        }
        if( salt != null ){
            item.setSalt(salt);
        }
        if( createDate != null ){
            item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( modifiedDate != null ){
            item.setModifiedDate(Tools.convertStringToDate(modifiedDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( role != null ){
            item.setRole(role);
        }
        if( statusCode != null ){
            item.setStatusCode(statusCode);
        }
        if( lastLoginDate != null ){
            item.setLastLoginDate(Tools.convertStringToDate(lastLoginDate, "yyyy-MM-dd HH:mm:ss"));
        }
        tSupervisorService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tSupervisorService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TSupervisor'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_supervisors/{t_supervisor_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTSupervisor(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_supervisor_id") Integer id
    ) {
        TSupervisor item = tSupervisorService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tSupervisorService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TSupervisor'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_supervisors/{t_supervisor_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTSupervisor(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_supervisor_id") Integer id,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "salt", required = false) String salt,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "modifiedDate", required = false) String modifiedDate,
            @RequestParam(value = "role", required = false) Integer role,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "lastLoginDate", required = false) String lastLoginDate
    ) {
        TSupervisor item = tSupervisorService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( name != null ){
            item.setName(name);
        }
        if( username != null ){
            item.setUsername(username);
        }
        if( password != null ){
            item.setPassword(password);
        }
        if( salt != null ){
            item.setSalt(salt);
        }
        if( createDate != null ){
            item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( modifiedDate != null ){
            item.setModifiedDate(Tools.convertStringToDate(modifiedDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( role != null ){
            item.setRole(role);
        }
        if( statusCode != null ){
            item.setStatusCode(statusCode);
        }
        if( lastLoginDate != null ){
            item.setLastLoginDate(Tools.convertStringToDate(lastLoginDate, "yyyy-MM-dd HH:mm:ss"));
        }
        tSupervisorService.update(item);

        return tSupervisorService.get(id);
    }

    @ApiOperation(value = "查询'TSupervisor'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_supervisors/{t_supervisor_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTSupervisor(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_supervisor_id") Integer id,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "salt", required = false) String salt,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "modifiedDate", required = false) String modifiedDate,
            @RequestParam(value = "role", required = false) Integer role,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "lastLoginDate", required = false) String lastLoginDate
    ) {
        if(request.getMethod().equals("GET")){
            TSupervisor item = tSupervisorService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TSupervisor item = tSupervisorService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( name != null ){
                item.setName(name);
            }
            if( username != null ){
                item.setUsername(username);
            }
            if( password != null ){
                item.setPassword(password);
            }
            if( salt != null ){
                item.setSalt(salt);
            }
            if( createDate != null ){
                item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( modifiedDate != null ){
                item.setModifiedDate(Tools.convertStringToDate(modifiedDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( role != null ){
                item.setRole(role);
            }
            if( statusCode != null ){
                item.setStatusCode(statusCode);
            }
            if( lastLoginDate != null ){
                item.setLastLoginDate(Tools.convertStringToDate(lastLoginDate, "yyyy-MM-dd HH:mm:ss"));
            }
            tSupervisorService.update(item);

            return tSupervisorService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TSupervisor item = tSupervisorService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tSupervisorService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TSupervisor'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_supervisors",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTSupervisors(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tSupervisorService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tSupervisorService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_supervisors/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTSupervisorsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TSupervisor> list = tSupervisorService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TSupervisor.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TSupervisor'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_supervisors/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTSupervisors(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TSupervisor> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TSupervisor item = new TSupervisor();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("name")   ){
            item.setName(jsonObject.get("name").toString());
        }
        if(jsonObject.containsKey("username")   ){
            item.setUsername(jsonObject.get("username").toString());
        }
        if(jsonObject.containsKey("password")   ){
            item.setPassword(jsonObject.get("password").toString());
        }
        if(jsonObject.containsKey("salt")   ){
            item.setSalt(jsonObject.get("salt").toString());
        }
        if(jsonObject.containsKey("createDate")   ){
            item.setCreateDate(Tools.convertStringToDate(jsonObject.get("createDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("modifiedDate")   ){
            item.setModifiedDate(Tools.convertStringToDate(jsonObject.get("modifiedDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("role")   ){
            item.setRole(Integer.valueOf(jsonObject.get("role").toString()));
        }
        if(jsonObject.containsKey("statusCode")   ){
            item.setStatusCode(Integer.valueOf(jsonObject.get("statusCode").toString()));
        }
        if(jsonObject.containsKey("lastLoginDate")   ){
            item.setLastLoginDate(Tools.convertStringToDate(jsonObject.get("lastLoginDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
             tSupervisorService.create(item);
             lists.add(tSupervisorService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TSupervisor'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_supervisors/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTSupervisor(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TSupervisor item = tSupervisorService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tSupervisorService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
