package com.haomostudio.so.utils;


import com.haomostudio.so.service.HmUtils.Tools;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.util.Date;
import java.util.UUID;

/**
 * Created by 123 on 2017/7/17.
 */
public class MyFileUtils {
    private MultipartFile multipartFile ;	// 用于进行上传文件的操作类对象
    /**
     * 实例化FileUtils程序类
     * @param multipartFile 所有的上传文件信息
     */
    public MyFileUtils(MultipartFile multipartFile) {
        this.multipartFile = multipartFile ;
    }

    public boolean backupFile(HttpServletRequest request, String fileName){
        boolean flag = false ;
        String parentDir = PropertiesUtil.getProperties("upload_file_path");

//        String parentDir = File.separator + dir ;
        File outFile = new File(parentDir,fileName) ;
        OutputStream output = null ;
        try {
            output = new FileOutputStream(outFile) ;
            InputStream input = this.multipartFile.getInputStream() ;
            int len = 0 ;	// 保存每次的读取数据长度
            byte [] data = new byte [2048] ;
            while ((len = input.read(data)) != -1) {
                output.write(data, 0, len);
            }
            flag = true ;
        } catch (Exception e) {
            e.printStackTrace();
            flag = false ;
        } finally {
            if (output != null) {
                try {
                    output.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return flag ;
    }

    /**
     * 如果要进行 文件的上传保存， 那么必须知道文件的存放父目录
     * 必须保证传入的保存路径存在
     * @param request request请求对象利用此对象获得ServletContext
     * @param dir 保存的目录，例如：“upload/emp/”；
     * @param fileName 要保存的文件名称
     * @return 成功返回true
     */
    public boolean saveFile(HttpServletRequest request, String dir, String fileName) {
        boolean flag = false ;
        String parentDir = request.getSession().getServletContext().getRealPath(File.separator + dir) ;
//        String parentDir = File.separator + dir ;
        File outFile = new File(parentDir) ;
        if (!outFile.exists()){
            outFile.mkdir();
        }
        outFile = new File(parentDir,fileName) ;
        OutputStream output = null ;
        try {
            output = new FileOutputStream(outFile) ;
            InputStream input = this.multipartFile.getInputStream() ;
            int len = 0 ;	// 保存每次的读取数据长度
            byte [] data = new byte [2048] ;
            while ((len = input.read(data)) != -1) {
                output.write(data, 0, len); ;
            }
            flag = true ;
        } catch (Exception e) {
            e.printStackTrace();
            flag = false ;
        } finally {
            if (output != null) {
                try {
                    output.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return flag ;
    }


    /**
     * 获取新的文件名称，该名称一定用于信息的保存
     * @return 文件名称
     */
    public String createFileName() {
//        System.out.println("****************** "+ this.multipartFile.getContentType());
        if ("application/vnd.openxmlformats-officedocument.wordprocessingml.document".equals(this.multipartFile.getContentType())) {
            return UUID.randomUUID() + ".doc" ;
        }
        return UUID.randomUUID() + "." + this.multipartFile.getContentType()
                .substring(this.multipartFile.getContentType().lastIndexOf("/") + 1);
    }

    /**
     * 根据传入的完整文件名，截取后缀
     * @param fileName 完整的文件名称
     * @return 随机生成的文件名称
     */
    public static synchronized String createFileName(String fileName) {
            String timeDate = Tools.convertDateToString(new Date(),"yyyyMMddHHmmssSSS") ;
            return timeDate + "_" + fileName ;
    }

    /**
     * 将数据库中的文件名转换为原始名称
     * @return 返回原始文件名称
     */
     public static String getOldName(String newName){
        return newName.substring(newName.indexOf("_") + 1);
    }
}
